=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-21 23:13:57 +0000
+++ bzrlib/branch.py	2005-09-25 05:01:17 +0000
@@ -24,8 +24,9 @@
      splitpath, \
      sha_file, appendpath, file_kind
 
+from bzrlib.store import copy_all
 from bzrlib.errors import BzrError, InvalidRevisionNumber, InvalidRevisionId, \
-     DivergedBranches, NotBranchError
+     DivergedBranches, NotBranchError, UnlistableStore, UnlistableBranch
 from bzrlib.textui import show_status
 from bzrlib.revision import Revision
 from bzrlib.delta import compare_trees
@@ -1331,7 +1332,7 @@
     return gen_file_id('TREE_ROOT')
 
 
-def copy_branch(branch_from, to_location, revno=None):
+def copy_branch(branch_from, to_location, revno=None, basis_branch=None):
     """Copy branch_from into the existing directory to_location.
 
     revision
@@ -1340,13 +1341,22 @@
 
     to_location
         The name of a local directory that exists but is empty.
+
+    revno
+        The revision to copy up to
+
+    basis_branch
+        A local branch to copy revisions from, related to branch_from
     """
     from bzrlib.merge import merge
 
     assert isinstance(branch_from, Branch)
+    print repr(to_location)
     assert isinstance(to_location, basestring)
     
     br_to = Branch.initialize(to_location)
+    if basis_branch is not None:
+        copy_stores(basis_branch, br_to)
     br_to.set_root_id(branch_from.get_root_id())
     if revno is None:
         revno = branch_from.revno()
@@ -1355,3 +1365,15 @@
           check_clean=False, ignore_zero=True)
     br_to.set_parent(branch_from.base)
     return br_to
+
+def copy_stores(branch_from, branch_to):
+    """Copies all entries from branch stores to another branch's stores.
+    """
+    store_pairs = ((branch_from.text_store,      branch_to.text_store),
+                   (branch_from.inventory_store, branch_to.inventory_store),
+                   (branch_from.revision_store,  branch_to.revision_store))
+    try:
+        for from_store, to_store in store_pairs: 
+            copy_all(from_store, to_store)
+    except UnlistableStore:
+        raise UnlistableBranch(from_store)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-23 03:22:16 +0000
+++ bzrlib/builtins.py	2005-09-25 05:01:17 +0000
@@ -374,10 +374,10 @@
     parameter, as in "branch foo/bar -r 5".
     """
     takes_args = ['from_location', 'to_location?']
-    takes_options = ['revision']
+    takes_options = ['revision', 'basis']
     aliases = ['get', 'clone']
 
-    def run(self, from_location, to_location=None, revision=None):
+    def run(self, from_location, to_location=None, revision=None, basis=None):
         from bzrlib.branch import copy_branch
         import tempfile
         import errno
@@ -398,6 +398,10 @@
                 else:
                     raise
             br_from.setup_caching(cache_root)
+            if basis is not None:
+                basis_branch = Branch.open_containing(basis)
+            else:
+                basis_branch = None
             if len(revision) == 1 and revision[0] is not None:
                 revno = revision[0].in_history(br_from)[0]
             else:
@@ -416,11 +420,13 @@
                 else:
                     raise
             try:
-                copy_branch(br_from, to_location, revno)
+                copy_branch(br_from, to_location, revno, basis_branch)
             except bzrlib.errors.NoSuchRevision:
                 rmtree(to_location)
                 msg = "The branch %s has no revision %d." % (from_location, revision[0])
                 raise BzrCommandError(msg)
+            except bzrlib.errors.UnlistableBranch:
+                msg = "The branch %s cannot be used as a --basis"
         finally:
             rmtree(cache_root)
 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-09-18 00:57:26 +0000
+++ bzrlib/commands.py	2005-09-25 05:01:17 +0000
@@ -352,6 +352,7 @@
 # the type.
 OPTIONS = {
     'all':                    None,
+    'basis':                  str,
     'diff-options':           str,
     'help':                   None,
     'file':                   unicode,

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-09-15 17:48:08 +0000
+++ bzrlib/errors.py	2005-09-25 05:01:17 +0000
@@ -143,3 +143,11 @@
     def __init__(self, branch):
         msg = "Branch %s has no commits." % branch
         BzrError.__init__(self, msg)
+
+class UnlistableStore(BzrError):
+    def __init__(self, store):
+        BzrError.__init__(self, "Store %s is not listable" % store)
+
+class UnlistableBranch(BzrError):
+    def __init__(self, br):
+        BzrError.__init__(self, "Stores for branch %s are not listable" % br)

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-23 03:22:16 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-25 05:01:17 +0000
@@ -29,6 +29,7 @@
 from cStringIO import StringIO
 import sys
 import os
+import shutil
 
 from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
 from bzrlib.branch import Branch
@@ -223,6 +224,13 @@
         os.chdir('..')
         self.runbzr('branch a b')
         self.runbzr('branch a c -r 1')
+        os.chdir('b')
+        self.runbzr('commit -m foo --unchanged')
+        os.chdir('..')
+        shutil.rmtree('a/.bzr/revision-store')
+        shutil.rmtree('a/.bzr/inventory-store')
+        shutil.rmtree('a/.bzr/text-store')
+        self.runbzr('branch a d --basis b')
 
     def test_merge(self):
         from bzrlib.branch import Branch

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-09-14 05:22:13 +0000
+++ bzrlib/selftest/testbranch.py	2005-09-25 05:01:17 +0000
@@ -14,20 +14,51 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
 from bzrlib.selftest import TestCaseInTempDir
-
-
-class TestAppendRevisions(TestCaseInTempDir):
-    """Test appending more than one revision"""
+from bzrlib.branch import Branch, copy_stores, copy_branch
+from bzrlib.commit import commit
+from bzrlib.errors import NoSuchRevision, UnlistableBranch
+
+class TestBranch(TestCaseInTempDir):
+
     def test_append_revisions(self):
-        from bzrlib.branch import Branch
+        """Test appending more than one revision"""
         br = Branch.initialize(".")
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
 
-
+    def test_copy_stores(self):
+        """Copy the stores from one branch to another"""
+        os.mkdir('a')
+        br_a = Branch.initialize("a")
+        file('a/b', 'wb').write('b')
+        br_a.add('b')
+        commit(br_a, "silly commit")
+
+        os.mkdir('b')
+        br_b = Branch.initialize("b")
+        self.assertRaises(NoSuchRevision, br_b.get_revision, 
+                          br_a.revision_history()[0])
+        copy_stores(br_a, br_b)
+        rev = br_b.get_revision(br_a.revision_history()[0])
+        tree = br_b.revision_tree(br_a.revision_history()[0])
+        for file_id in tree:
+            if tree.inventory[file_id].kind == "file":
+                tree.get_file(file_id).read()
+        return br_a, br_b
+
+    def test_copy_branch(self):
+        """Copy the stores from one branch to another"""
+        br_a, br_b = self.test_copy_stores()
+        commit(br_b, "silly commit")
+        os.mkdir('c')
+        br_c = copy_branch(br_a, 'c', basis_branch=br_b)
+        self.assertEqual(br_a.revision_history(), br_c.revision_history())
+        assert br_b.last_patch() not in br_c.revision_history()
+        br_c.get_revision(br_b.last_patch())
 # TODO: rewrite this as a regular unittest, without relying on the displayed output        
 #         >>> from bzrlib.commit import commit
 #         >>> bzrlib.trace.silent = True

=== modified file 'bzrlib/selftest/teststore.py'
--- bzrlib/selftest/teststore.py	2005-09-04 02:59:56 +0000
+++ bzrlib/selftest/teststore.py	2005-09-25 05:01:17 +0000
@@ -16,10 +16,12 @@
 
 """Test Store implementation
 """
-from bzrlib.store import ImmutableStore
+from bzrlib.store import ImmutableStore, copy_all
+from bzrlib.remotebranch import RemoteStore
 from bzrlib.selftest import TestCaseInTempDir
 from StringIO import StringIO
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, UnlistableStore
+import os
 
 class TestStore(TestCaseInTempDir):
     def test_multiple_add(self):
@@ -28,6 +30,20 @@
         store.add(StringIO('goodbye'), '123123')
         self.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
 
+    def test_copy_all(self):
+        """Test copying"""
+        os.mkdir('a')
+        store_a = ImmutableStore('a')
+        store_a.add('foo', '1')
+        os.mkdir('b')
+        store_b = ImmutableStore('b')
+        copy_all(store_a, store_b)
+        self.assertEqual(store_a['1'].read(), 'foo')
+        self.assertEqual(store_b['1'].read(), 'foo')
+        store_c = RemoteStore('http://example.com/')
+        self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+        
+
 TEST_CLASSES = [
     TestStore,
     ]

=== modified file 'bzrlib/store.py'
--- bzrlib/store.py	2005-09-16 03:32:44 +0000
+++ bzrlib/store.py	2005-09-25 05:01:17 +0000
@@ -24,7 +24,7 @@
 import os, tempfile, types, osutils, gzip, errno
 from stat import ST_SIZE
 from StringIO import StringIO
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, UnlistableStore
 from bzrlib.trace import mutter
 import bzrlib.ui
 
@@ -254,3 +254,10 @@
             os.remove(fpath)
         os.rmdir(self._basedir)
         mutter("%r destroyed" % self)
+
+def copy_all(store_from, store_to):
+    """Copy all ids from one store to another."""
+    if not hasattr(store_from, "__iter__"):
+        raise UnlistableStore(store_from)
+    ids = [f for f in store_from]
+    store_to.copy_multi(store_from, ids)

